#ifndef __COLOR_H
#define __COLOR

/* Text attributes */
#define ESC     "\033[0m"
#define BOLD    "\033[1m"
#define UNDER   "\033[4m"
#define BLINK   "\033[5m"

/* Foreground colors */
#define BLACK   "\033[30m"
#define RED     "\033[31m"
#define GREEN   "\033[32m"
#define YELLOW  "\033[33m"
#define BLUE    "\033[34m"
#define MAGENTA "\033[35m"
#define CYAN    "\033[36m"
#define WHITE   "\033]37m"

/* Background colors*/
#define BBLACK   "\033[40m"
#define BRED     "\033[41m"
#define BGREEN   "\033[42m"
#define BYELLOW  "\033[43m"
#define BBLUE    "\033[44m"
#define BAGENTA  "\033[45m"
#define BCYAN    "\033[46m"
#define BWHITE   "\033]47m"


#endif